@@ -7,14 +7,8 @@
 package yara
 
 /*
-#ifdef _WIN32
-#define fdopen _fdopen
-#define dup _dup
-#endif
-#include <stdio.h>
-#include <unistd.h>
-
 #include <yara.h>
+#include "compat.h"
 
 void compilerCallback(int, char*, int, YR_RULE*, char*, void*);
 char* includeCallback(char*, char*, char*, void*);
@@ -31,12 +25,17 @@ import (
 
 //export compilerCallback
 func compilerCallback(errorLevel C.int, filename *C.char, linenumber C.int, rule *C.YR_RULE, message *C.char, userData unsafe.Pointer) {
-	c := callbackData.Get(userData).(*Compiler)
+	c := cgoHandle(userData).Value().(*Compiler)
 	msg := CompilerMessage{
 		Filename: C.GoString(filename),
 		Line:     int(linenumber),
 		Text:     C.GoString(message),
 	}
+	if rule != nil {
+		// Rule object implicitly relies on the compiler object and is destroyed when the compiler is destroyed.
+		// Save a reference to the compiler to prevent that from happening.
+		msg.Rule = &Rule{cptr: rule, owner: c}
+	}
 	switch errorLevel {
 	case C.YARA_ERROR_LEVEL_ERROR:
 		c.Errors = append(c.Errors, msg)
@@ -55,7 +54,7 @@ type Compiler struct {
 	Errors   []CompilerMessage
 	Warnings []CompilerMessage
 	// used for include callback
-	callbackData unsafe.Pointer
+	callbackData cgoHandle
 	cptr         *C.YR_COMPILER
 }
 
@@ -65,6 +64,7 @@ type CompilerMessage struct {
 	Filename string
 	Line     int
 	Text     string
+	Rule     *Rule
 }
 
 // NewCompiler creates a YARA compiler.
@@ -89,9 +89,9 @@ func (c *Compiler) Destroy() {
 	runtime.SetFinalizer(c, nil)
 }
 
-func (c *Compiler) setCallbackData(ptr unsafe.Pointer) {
-	if c.callbackData != nil {
-		callbackData.Delete(c.callbackData)
+func (c *Compiler) setCallbackData(ptr cgoHandle) {
+	if c.callbackData != 0 {
+		c.callbackData.Delete()
 	}
 	c.callbackData = ptr
 }
@@ -112,10 +112,10 @@ func (c *Compiler) AddFile(file *os.File, namespace string) (err error) {
 	}
 	filename := C.CString(file.Name())
 	defer C.free(unsafe.Pointer(filename))
-	id := callbackData.Put(c)
-	defer callbackData.Delete(id)
-	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), id)
-	numErrors := int(C.yr_compiler_add_fd(c.cptr, (C.YR_FILE_DESCRIPTOR)(file.Fd()), ns, filename))
+	id := cgoNewHandle(c)
+	defer id.Delete()
+	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), unsafe.Pointer(id))
+	numErrors := int(C._yr_compiler_add_fd(c.cptr, C.int(file.Fd()), ns, filename))
 	if numErrors > 0 {
 		var buf [1024]C.char
 		msg := C.GoString(C.yr_compiler_get_error_message(
@@ -142,9 +142,9 @@ func (c *Compiler) AddString(rules string, namespace string) (err error) {
 	}
 	crules := C.CString(rules)
 	defer C.free(unsafe.Pointer(crules))
-	id := callbackData.Put(c)
-	defer callbackData.Delete(id)
-	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), id)
+	id := cgoNewHandle(c)
+	defer id.Delete()
+	C.yr_compiler_set_callback(c.cptr, C.YR_COMPILER_CALLBACK_FUNC(C.compilerCallback), unsafe.Pointer(id))
 	numErrors := int(C.yr_compiler_add_string(c.cptr, crules, ns))
 	if numErrors > 0 {
 		var buf [1024]C.char
@@ -205,7 +205,7 @@ func (c *Compiler) GetRules() (*Rules, error) {
 
 //export includeCallback
 func includeCallback(name, filename, namespace *C.char, userData unsafe.Pointer) *C.char {
-	callbackFunc := callbackData.Get(userData).(CompilerIncludeFunc)
+	callbackFunc := cgoHandle(userData).Value().(CompilerIncludeFunc)
 	if buf := callbackFunc(
 		C.GoString(name), C.GoString(filename), C.GoString(namespace),
 	); buf != nil {
@@ -249,13 +249,13 @@ func (c *Compiler) SetIncludeCallback(cb CompilerIncludeFunc) {
 		c.DisableIncludes()
 		return
 	}
-	id := callbackData.Put(cb)
+	id := cgoNewHandle(cb)
 	c.setCallbackData(id)
 	C.yr_compiler_set_include_callback(
 		c.cptr,
 		C.YR_COMPILER_INCLUDE_CALLBACK_FUNC(C.includeCallback),
 		C.YR_COMPILER_INCLUDE_FREE_FUNC(C.freeCallback),
-		id,
+		unsafe.Pointer(id),
 	)
 	runtime.KeepAlive(c)
 	return
@@ -265,7 +265,7 @@ func (c *Compiler) SetIncludeCallback(cb CompilerIncludeFunc) {
 // See yr_compiler_set_include_callbacks.
 func (c *Compiler) DisableIncludes() {
 	C.yr_compiler_set_include_callback(c.cptr, nil, nil, nil)
-	c.setCallbackData(nil)
+	c.setCallbackData(0)
 	runtime.KeepAlive(c)
 	return
 }
