@@ -64,6 +64,12 @@ type ScanCallbackModuleImportFinished interface {
 	ModuleImported(*ScanContext, *Object) (bool, error)
 }
 
+/// ScanCAllbackConsoleLog can be used to implement custom functions
+/// to handle the console.log feature introduced in YARA 4.2.
+type ScanCallbackConsoleLog interface {
+	ConsoleLog(*ScanContext, string)
+}
+
 // scanCallbackContainer is used by to pass a ScanCallback (and
 // associated data) between ScanXxx methods and scanCallbackFunc(). It
 // stores the public callback interface and a list of malloc()'d C
@@ -97,7 +103,7 @@ func (c *scanCallbackContainer) finalize() {
 
 //export scanCallbackFunc
 func scanCallbackFunc(ctx *C.YR_SCAN_CONTEXT, message C.int, messageData, userData unsafe.Pointer) C.int {
-	cbc, ok := callbackData.Get(userData).(*scanCallbackContainer)
+	cbc, ok := cgoHandle(userData).Value().(*scanCallbackContainer)
 	s := &ScanContext{cptr: ctx}
 	if !ok {
 		return C.CALLBACK_ERROR
@@ -137,6 +143,10 @@ func scanCallbackFunc(ctx *C.YR_SCAN_CONTEXT, message C.int, messageData, userDa
 		if c, ok := cbc.ScanCallback.(ScanCallbackModuleImportFinished); ok {
 			abort, err = c.ModuleImported(s, &Object{(*C.YR_OBJECT)(messageData)})
 		}
+	case C.CALLBACK_MSG_CONSOLE_LOG:
+		if c, ok := cbc.ScanCallback.(ScanCallbackConsoleLog); ok {
+			c.ConsoleLog(s, C.GoString((*C.char)(messageData)))
+		}
 	}
 
 	if err != nil {
