@@ -8,18 +8,7 @@ package yara
 
 /*
 #include <yara.h>
-
-#ifdef _WIN32
-#include <stdint.h>
-int _yr_scanner_scan_fd(
-    YR_SCANNER* scanner,
-    int fd)
-{
-  return yr_scanner_scan_fd(scanner, (YR_FILE_DESCRIPTOR)(intptr_t)fd);
-}
-#else
-#define _yr_scanner_scan_fd yr_scanner_scan_fd
-#endif
+#include "compat.h"
 
 int scanCallbackFunc(YR_SCAN_CONTEXT*, int, void*, void*);
 */
@@ -127,16 +116,15 @@ func (s *Scanner) SetCallback(cb ScanCallback) *Scanner {
 }
 
 // putCallbackData stores the scanner's callback object in
-// callbackData, returning a pointer. If no callback object has been
+// a cgoHandle. If no callback object has been
 // set, it is initialized with the pointer to an empty ScanRules
-// object. The object must be removed from callbackData by the calling
-// ScanXxxx function.
-func (s *Scanner) putCallbackData() unsafe.Pointer {
+// object. The handle must be deleted by the calling ScanXxxx function.
+func (s *Scanner) putCallbackData() cgoHandle {
 	if _, ok := s.Callback.(ScanCallback); !ok {
 		s.Callback = &MatchRules{}
 	}
-	ptr := callbackData.Put(makeScanCallbackContainer(s.Callback, s.rules))
-	C.yr_scanner_set_callback(s.cptr, C.YR_CALLBACK_FUNC(C.scanCallbackFunc), ptr)
+	ptr := cgoNewHandle(makeScanCallbackContainer(s.Callback, s.rules))
+	C.yr_scanner_set_callback(s.cptr, C.YR_CALLBACK_FUNC(C.scanCallbackFunc), unsafe.Pointer(ptr))
 	return ptr
 }
 
@@ -151,7 +139,7 @@ func (s *Scanner) ScanMem(buf []byte) (err error) {
 	}
 
 	cbPtr := s.putCallbackData()
-	defer callbackData.Delete(cbPtr)
+	defer cbPtr.Delete()
 
 	C.yr_scanner_set_flags(s.cptr, s.flags.withReportFlags(s.Callback))
 	err = newError(C.yr_scanner_scan_mem(
@@ -171,7 +159,7 @@ func (s *Scanner) ScanFile(filename string) (err error) {
 	defer C.free(unsafe.Pointer(cfilename))
 
 	cbPtr := s.putCallbackData()
-	defer callbackData.Delete(cbPtr)
+	defer cbPtr.Delete()
 
 	C.yr_scanner_set_flags(s.cptr, s.flags.withReportFlags(s.Callback))
 	err = newError(C.yr_scanner_scan_file(
@@ -188,7 +176,7 @@ func (s *Scanner) ScanFile(filename string) (err error) {
 // SetCAllback, it is initialized with an empty MatchRules object.
 func (s *Scanner) ScanFileDescriptor(fd uintptr) (err error) {
 	cbPtr := s.putCallbackData()
-	defer callbackData.Delete(cbPtr)
+	defer cbPtr.Delete()
 
 	C.yr_scanner_set_flags(s.cptr, s.flags.withReportFlags(s.Callback))
 	err = newError(C._yr_scanner_scan_fd(
@@ -205,7 +193,7 @@ func (s *Scanner) ScanFileDescriptor(fd uintptr) (err error) {
 // SetCAllback, it is initialized with an empty MatchRules object.
 func (s *Scanner) ScanProc(pid int) (err error) {
 	cbPtr := s.putCallbackData()
-	defer callbackData.Delete(cbPtr)
+	defer cbPtr.Delete()
 
 	C.yr_scanner_set_flags(s.cptr, s.flags.withReportFlags(s.Callback))
 	err = newError(C.yr_scanner_scan_proc(
@@ -224,10 +212,10 @@ func (s *Scanner) ScanMemBlocks(mbi MemoryBlockIterator) (err error) {
 	c := makeMemoryBlockIteratorContainer(mbi)
 	defer c.free()
 	cmbi := makeCMemoryBlockIterator(c)
-	defer callbackData.Delete(cmbi.context)
+	defer cgoHandle(cmbi.context).Delete()
 
 	cbPtr := s.putCallbackData()
-	defer callbackData.Delete(cbPtr)
+	defer cbPtr.Delete()
 
 	C.yr_scanner_set_flags(s.cptr, s.flags.withReportFlags(s.Callback))
 	err = newError(C.yr_scanner_scan_mem_blocks(
